import {
	ButtonStyles,
	ChannelTypes,
	ComponentTypes,
	MessageFlags,
	Shard
} from "oceanic.js";
import { client } from ".";
import { Constants } from "./Constants";
import { db } from "./utils/database";
import {
	ActionRow,
	Button,
	ComponentMessage,
	Container,
	Divider,
	Section,
	Separator,
	TextDisplay
} from "~/components";
import { selfappReq } from "./utils/selfappReq";
import { User } from "./components/User";
import { Response } from "./utils/types";
import { match } from "./utils/utils";
import { PendingApplicationMessage } from "./components/PendingApplicationMessage";
import { ApplicationContent } from "./components/ApplicationContent";

import("./joinRequestActions/lockUnlock");
import("./joinRequestActions/interview");
import("./joinRequestActions/accept");
import("./joinRequestActions/reject");

export async function setupJoinRequestHandler(shard: Shard) {
	shard.ws?.on("message", async (d) => {
		const data = JSON.parse(d.toString("utf8"));
		if (!data.t) return;
		if (data.t.startsWith("GUILD_JOIN_REQUEST")) {
			const applicationData = data.d;
			switch (data.t) {
				case "GUILD_JOIN_REQUEST_UPDATE": {
					switch (applicationData.status) {
						case "SUBMITTED": {
							const rawResponses =
								applicationData.request.form_responses;
							const response: Response = {
								hereFor: match(rawResponses[0].response, {
									0: ["tag"],
									1: ["chat"],
									2: ["tag", "chat"]
								}),
								foundThrough: match(rawResponses[1].response, {
									0: "archive",
									1: "user",
									2: "tag",
									3: "other",
									4: "sold"
								}),
								languages: rawResponses[2].response,
								details: rawResponses[3].response,
								canBoost: rawResponses[4].response === 0
							};
							const user = await client.rest.users.get(
								applicationData.request.user.id
							);
							const baller = {
								name: `${user.tag}'s application`,
								message: (
									<PendingApplicationMessage
										id={applicationData.request.id}
										response={response}
										user={user}
										locked={
											response.foundThrough === "sold"
										}
									/>
								),
								appliedTags: [Constants.TAGS.PENDING]
							};
							console.log(JSON.stringify(baller));
							const thread =
								await client.rest.channels.startThreadInThreadOnlyChannel(
									Constants.APPLICATIONS_CHANNEL_ID,
									baller
								);

							const fullApplicantProfile: any = await selfappReq(
								`/users/${user.id}/profile`,
								"GET",
								{
									join_request_id: applicationData.request.id
								}
							);

							thread.createMessage({
								content: `<@&${Constants.REVIEWER_ROLE_ID}> new application from \`@${user.tag}\``,
								embeds: [
									{
										color: Constants.COLORS.NEW_REQ,
										title: "User bio",
										description:
											"```\n" +
											fullApplicantProfile.user_profile
												.bio +
											"\n```"
									}
								]
							});

							await db.run(
								"INSERT INTO applications (id, user_id, status, message_id, channel_id, responses, thread_id) VALUES (?, ?, 0, ?, ?, ?, ?)",
								applicationData.request.id,
								applicationData.request.user_id,
								thread.id,
								thread.id,
								JSON.stringify(response),
								thread.id
							);

							break;
						}
					}
					break;
				}
				case "GUILD_JOIN_REQUEST_DELETE": {
					const application = await db.get(
						"SELECT * FROM applications WHERE id=?",
						applicationData.id
					);
					const user = await client.rest.users.get(
						applicationData.user_id
					);

					const response: Response = JSON.parse(
						application ? application.responses : "{}" // this will never be used if no applications
					);

					if (application) {
						await db.run(
							"DELETE FROM applications WHERE id=?",
							applicationData.id
						);
						if (application.status === 1) return;
						client.rest.channels.editMessage(
							application.thread_id,
							application.message_id,
							<ComponentMessage>
								<Container accentColor={Constants.COLORS.BAD}>
									<TextDisplay>
										## Join request withdrawn
									</TextDisplay>
									<Divider />
									<User user={user} />
									<Divider />
									<ApplicationContent
										response={response}
										includeDetails={true}
									/>
									{application.reject_reason && <Divider />}
									{application.reject_reason && (
										<TextDisplay>
											### Rejection reason
											<br />
											{application.reject_reason}
										</TextDisplay>
									)}
								</Container>
							</ComponentMessage>
						);
						await client.rest.channels.edit(application.thread_id, {
							archived: true,
							locked: true,
							reason: `${user.tag}'s application has been withdrawn`,
							appliedTags: [Constants.TAGS.WITHDRAWN]
						});
						application.reject_reason &&
							client.rest.channels.edit(application.thread_id, {
								appliedTags: [
									Constants.TAGS.REJECTED,
									Constants.TAGS.WITHDRAWN
								]
							});
					} else {
						const th =
							await client.rest.channels.startThreadInThreadOnlyChannel(
								Constants.APPLICATIONS_CHANNEL_ID,
								{
									name: `${user.tag}'s application`,
									message: (
										<ComponentMessage>
											<Container
												accentColor={
													Constants.COLORS.BAD
												}
											>
												<TextDisplay>
													## Join request withdrawn
												</TextDisplay>
												<Divider />
												<User user={user} />
												<Divider />
												<TextDisplay>
													### Application
													<br />
													User hasn't applied
												</TextDisplay>
											</Container>
										</ComponentMessage>
									),
									appliedTags: [Constants.TAGS.WITHDRAWN]
								}
							);
						th.edit({
							locked: true
						});
					}
					break;
				}
			}
		}
	});
}
